local HBDPin = LibStub("HereBeDragons-Pins-1.0", true)

local TIME_LOST_MIRROR_SPELL_ID = 194812
local TIME_LOST_MIRROR_SPELL_NAME = GetSpellInfo(TIME_LOST_MIRROR_SPELL_ID)

local AddMarkers
local RemoveMarkers
do
	local DRAENOR = 962
	local OUTLAND = 466

	local db = {
		{ OUTLAND, 0.2852, 0.6832, "Oshugun Spirit Woods" },     { DRAENOR, 0.2547, 0.5565, "Oshugun Spirit Fields" },
		{ OUTLAND, 0.3432, 0.4932, "North-East Nagrand Coast" }, { DRAENOR, 0.3316, 0.4361, "Twinspire Ruins" },
		{ OUTLAND, 0.3456, 0.5773, "Throne of the Elements" },   { DRAENOR, 0.3073, 0.4684, "Throne of the Elements" },
		{ OUTLAND, 0.3608, 0.3618, "Gormaul Tower" },            { DRAENOR, 0.3092, 0.3558, "Bloodmaul Ravine" },
		{ OUTLAND, 0.3818, 0.3203, "Burning Glacier" },          { DRAENOR, 0.2670, 0.3144, "Bloodmaul Ravine" },
		{ OUTLAND, 0.3979, 0.5885, "Nagrand Border" },           { DRAENOR, 0.3496, 0.4707, "Entrance to Nagrand" },
		{ OUTLAND, 0.4213, 0.3441, "Razor Bloom" },              { DRAENOR, 0.4933, 0.3707, "Razor Ridge" },
		{ OUTLAND, 0.4394, 0.5242, "Path of Glory" },            { DRAENOR, 0.4613, 0.4707, "Zangarmarsh Border" },
		{ OUTLAND, 0.4433, 0.2032, "Blackrock Foundry" },        { DRAENOR, 0.4984, 0.2188, "Gruul's Lair" },
		{ OUTLAND, 0.4476, 0.6246, "Shattrath" },                { DRAENOR, 0.4141, 0.5385, "Shattrath" },
		{ OUTLAND, 0.4788, 0.7329, "Deathweb Hollow" },          { DRAENOR, 0.4336, 0.6574, "Bone Wastes" },
		{ OUTLAND, 0.5574, 0.8205, "Talador Border" },           { DRAENOR, 0.4589, 0.6804, "Skettis" },
		{ OUTLAND, 0.5876, 0.7724, "Moonflower Valley" },        { DRAENOR, 0.5339, 0.6477, "Legion Hold" },
		{ OUTLAND, 0.5895, 0.5174, "Hellfire Citadel" },         { DRAENOR, 0.5778, 0.4641, "Magtheridon's Lair" },
		{ OUTLAND, 0.6164, 0.4371, "Throne of Kil'jaeden" },     { DRAENOR, 0.5934, 0.4090, "Throne of Kil'jaeden" },
		{ OUTLAND, 0.6647, 0.5254, "Dark Portal" },              { DRAENOR, 0.6257, 0.4729, "Dark Portal" },
		{ OUTLAND, 0.6960, 0.8124, "Path of Light" },            { DRAENOR, 0.6155, 0.7055, "The Warden's Cage" },
	}

	local lastClicked

	local function OnClick(icon)
		if not icon.marker.target then return end
		local areaID = GetCurrentMapAreaID()
		if areaID == DRAENOR or areaID == OUTLAND then
			SetMapByID(icon.marker.target[1])
		else
			if not icon.marker.target[5] then return end
			SetMapByID(icon.marker.target[5])
		end
		if lastClicked then
			lastClicked.arrow:Hide()
			lastClicked.texture1:Show()
			lastClicked.texture2:Hide()
		end
		lastClicked = icon.marker.target.icon1
		lastClicked.texture1:Hide()
		lastClicked.texture2:Show()
		lastClicked.arrow:Show()
	end

	local function OnEnter(icon)
		if not icon.tooltip then return end
		WorldMapTooltip:SetOwner(icon, "ANCHOR_TOPLEFT")
		WorldMapTooltip:SetText(icon.tooltip)
		WorldMapTooltip:Show()
	end

	local function OnLeave(icon)
		WorldMapTooltip:Hide()
	end

	local function Animation_OnFinished(animation)
		animation.icon.arrow:ClearAllPoints()
		animation.up = not animation.up

		if animation.up then
			animation.icon.arrow:SetPoint("BOTTOM", animation.icon, "TOP", 0, 10)
			animation.bounce:SetSmoothing("OUT")
		else
			animation.icon.arrow:SetPoint("BOTTOM", animation.icon, "TOP", 0, 0)
			animation.bounce:SetSmoothing("IN")
		end

		animation.bounce:SetOffset(0, animation.up and -10 or 10)
		animation:Play()
	end

	local function CreateIcon()
		local icon = CreateFrame("Button", nil, WorldMapButton)
		icon:Hide()
		icon:SetFrameStrata("HIGH")
		icon:SetFrameLevel(999)
		icon:SetSize(20, 20)
		icon:EnableMouse(true)
		icon:RegisterForClicks("LeftButtonUp")
		icon:SetScript("OnClick", OnClick)
		icon:SetScript("OnEnter", OnEnter)
		icon:SetScript("OnLeave", OnLeave)
		icon.texture1 = icon:CreateTexture(nil, "BACKGROUND")
		icon.texture1:SetPoint("CENTER", 0, 0)
		icon.texture1:SetAtlas("legioninvasion-map-icon-portal-large")
		icon.texture1:SetSize(77*.5, 81*.5)
		icon.texture2 = icon:CreateTexture(nil, "BACKGROUND")
		icon.texture2:SetPoint("CENTER", 0, 0)
		icon.texture2:SetAtlas("legioninvasion-map-icon-portal")
		icon.texture2:SetSize(77*.6, 81*.6)
		icon.texture2:Hide()
		icon.text = icon:CreateFontString(nil, nil, "GameTooltipText")
		icon.text:SetFont(icon.text:GetFont(), 24, "OUTLINE")
		icon.text:SetPoint("LEFT", icon, "RIGHT", 2, 0)
		icon.arrow = icon:CreateTexture(nil, "OVERLAY")
		icon.arrow:SetPoint("BOTTOM", icon, "TOP", 0, 0)
		icon.arrow:SetSize(32, 32)
		icon.arrow:SetTexture("Interface\\Minimap\\Minimap-DeadArrow")
		icon.arrow:SetTexCoord(0, 1, 1, 0)
		icon.arrow.animation = icon.arrow:CreateAnimationGroup()
		icon.arrow.animation.bounce = icon.arrow.animation:CreateAnimation("Translation")
		icon.arrow.animation.bounce:SetOffset(0, 10)
		icon.arrow.animation.bounce:SetDuration(.5)
		icon.arrow.animation.bounce:SetSmoothing("IN")
		icon.arrow.animation.icon = icon
		icon.arrow.animation:SetScript("OnFinished", Animation_OnFinished)
		icon.arrow.animation:Play()
		icon.arrow:Hide()
		return icon
	end

	for i = 1, #db do
		local marker = db[i]

		marker[5] = C_MapCanvas.FindZoneAtPosition(marker[1], marker[2], marker[3])

		marker.icon1 = CreateIcon()
		marker.icon1.marker = marker
		-- marker.icon1.text:SetText(marker[4])

		marker.icon2 = CreateIcon()
		marker.icon2.texture1:SetSize(77*.3, 81*.3)
		marker.icon2.texture2:SetSize(77*.2, 81*.2)
	end

	for i = 1, #db, 2 do
		local a, b = db[i], db[i + 1]
		a.target = b
		b.target = a

		a.icon1.tooltip = "|cff999999To|r |cffFFFFFF" .. b[4] .. "|r"
		b.icon1.tooltip = "|cff999999To|r |cffFFFFFF" .. a[4] .. "|r"
	end

	function AddMarkers()
		for i = 1, #db do
			local marker = db[i]

			HBDPin:AddWorldMapIconMF(marker, marker.icon1, marker[1], 0, marker[2], marker[3])
			HBDPin:AddMinimapIconMF(marker, marker.icon2, marker[1], 0, marker[2], marker[3], false)
		end
	end

	function RemoveMarkers()
		for i = 1, #db do
			local marker = db[i]

			HBDPin:RemoveWorldMapIcon(marker, marker.icon1)
			HBDPin:RemoveMinimapIcon(marker, marker.icon2)
		end

		if lastClicked then
			lastClicked.arrow:Hide()
			lastClicked.texture1:Show()
			lastClicked.texture2:Hide()
			lastClicked = nil
		end
	end
end

local frame = CreateFrame("Frame")

frame:SetScript("OnEvent", function(frame, event, _, _, _, _, spellID)
	if event == "UNIT_SPELLCAST_SUCCEEDED" then
		if spellID == TIME_LOST_MIRROR_SPELL_ID then
			frame:RegisterUnitEvent("UNIT_AURA", "player")
			AddMarkers()
		end
	elseif event == "UNIT_AURA" then
		if not UnitBuff("player", TIME_LOST_MIRROR_SPELL_NAME) then
			frame:UnregisterEvent("UNIT_AURA")
			RemoveMarkers()
		end
	end
end)

frame:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "player")
